<?php

class ControllerApiFurgonetka extends Controller
{
    const INVALID_CREDENTIALS_ERROR = "Invalid credentials";

    public function index()
    {
        $this->load->model('account/api');
        $this->load->model('account/custom_field');
        $this->load->model('checkout/order');
        $json = [];
        $limit = 50;
        $datetime = date('Y-m-d H:i:s');

        if (isset($this->request->post['limit'])) {
            $limit = (int)$this->request->post['limit'];
            if ($limit > 50) {
                $limit = 50;
            }
        }
        if (isset($this->request->post['datetime'])) {
            $datetime = $this->request->post['datetime'];
        }

        if (isset($this->request->post['key'])) {
            $api_info = $this->model_account_api->login('Furgonetka', $this->request->post['key']);

            if ($api_info) {
                $ip_data = [];
                $ip = $this->request->server['REMOTE_ADDR'];
                $api_id = $api_info['api_id'];

                $results = $this->model_account_api->getApiIps($api_info['api_id']);

                foreach ($results as $result) {
                    $ip_data[] = trim($result['ip']);
                }

                if (!in_array($ip, $ip_data)) {
                    $this->db->query("INSERT INTO `" . DB_PREFIX . "api_ip` (api_id, ip) VALUES ('$api_id', '$ip')");
                }
                $session = new Session($this->config->get('session_engine'), $this->registry);
                $session->start();
                $this->model_account_api->addApiSession(
                    $api_info['api_id'],
                    $session->getId(),
                    $this->request->server['REMOTE_ADDR']
                );
                $session->data['api_id'] = $api_info['api_id'];
                $json['orders'] = [];
                $column = $this->db->query("select column_name from INFORMATION_SCHEMA.columns where table_name = '" . DB_PREFIX . "order' and column_name = 'parcelLocker'");
                $fields = 'order_id';
                if (count($column->rows) > 0) {
                    $fields .= ', parcelLocker';
                }
                $orders = $this->db->query(
                    "SELECT " . $fields . " FROM `" . DB_PREFIX . "order` WHERE date_modified>'$datetime' order by date_modified limit $limit"
                );

                foreach ($orders->rows as $order) {
                    $orderData = $this->model_checkout_order->getOrder($order['order_id']);
                    $orderProducts = $this->model_checkout_order->getOrderProducts($order['order_id']);
                    $orderTotals = $this->model_checkout_order->getOrderTotals($order['order_id']);
                    $orderProductsArray = [];
                    $codAmount = 0.0;

                    foreach ($orderProducts as $orderProduct) {
                        $orderProductsArray[] = [
                            'sourceProductId' => $orderProduct['product_id'],
                            'name' => $orderProduct['name'] . ' ' . $orderProduct['model'],
                            'priceGross' => round($orderProduct['price'] + $orderProduct['tax'], 2),
                            'priceNet' => round($orderProduct['price'], 2),
                            'quantity' => $orderProduct['quantity'],
                            'tax' => $this->getTaxRate($orderProduct['product_id'], $orderProduct['price'], $orderData),
                        ];
                    }
                    $priceTotal = round($orderData['total'], 2);

                    if ($orderData['payment_code'] == 'cod') {
                        $codAmount = $priceTotal;
                    }
                    $shippingCost = null;
                    foreach ($orderTotals as $orderTotal) {
                        if ($orderTotal['code'] === 'shipping') {
                            $shippingCost = round($orderTotal['value'], 2);
                        }
                    }
                    
                    $shippingAddress = $orderData['shipping_address_1'];
                    if (!empty($orderData['shipping_address_2'])) {
                        $shippingAddress .= ' ' . $orderData['shipping_address_2'];
                    }

                    $paymentAddress = $orderData['payment_address_1'];
                    if (!empty($orderData['payment_address_2'])) {
                        $paymentAddress .= ' ' . $orderData['payment_address_2'];
                    }

                    $json['orders'][] = [
                        'sourceOrderId' => $orderData['order_id'],
                        'sourceClientId' => $orderData['customer_id'],
                        'datetimeOrder' => $orderData['date_added'],
                        'sourceDatetimeChange' => $orderData['date_modified'],
                        'status' => $orderData['order_status'],
                        'currency' => $orderData['currency_code'],
                        'totalPrice' => $priceTotal,
                        'codAmount' => $codAmount,
                        'comment' => $orderData['comment'],
                        'serviceDescription' => $orderData['shipping_method'],
                        'paymentMethod' => $orderData['payment_method'],
                        'shippingCost' => $shippingCost,
                        'shippingAddress' => [
                            'company' => $orderData['shipping_company'],
                            'name' => $orderData['shipping_firstname'],
                            'surname' => $orderData['shipping_lastname'],
                            'street' => $shippingAddress,
                            'city' => $orderData['shipping_city'],
                            'postcode' => $orderData['shipping_postcode'],
                            'countryCode' => $orderData['shipping_iso_code_2'],
                            'county' => $orderData['shipping_zone'],
                            'phone' => $orderData['telephone'],
                            'email' => $orderData['email'],
                            'taxNumber' => $this->parseTaxNumber($orderData['shipping_custom_field']),
                        ],
                        'paymentAddress' => [
                            'company' => $orderData['payment_company'],
                            'name' => $orderData['payment_firstname'],
                            'surname' => $orderData['payment_lastname'],
                            'street' => $paymentAddress,
                            'city' => $orderData['payment_city'],
                            'postcode' => $orderData['payment_postcode'],
                            'countryCode' => $orderData['payment_iso_code_2'],
                            'county' => $orderData['payment_zone'],
                            'taxNumber' => $this->parseTaxNumber($orderData['payment_custom_field']),
                        ],
                        'products' => $orderProductsArray,
                        'point' => isset($order['parcelLocker']) ? $order['parcelLocker'] : null,
                    ];
                }
            } else {
                $json['error'] = self::INVALID_CREDENTIALS_ERROR;
            }
        } else {
            $json['error'] = self::INVALID_CREDENTIALS_ERROR;
        }

        $this->response->addHeader('Content-Type: application/json');
        $this->response->setOutput(json_encode($json));
    }

    private function getTaxRate($productId, $productPrice, $order)
    {
        $taxRates = $this->db->query(
            "SELECT tr.tax_rate_id, tr.based, p.tax_class_id FROM `" . DB_PREFIX . "tax_rule` tr LEFT JOIN " . DB_PREFIX . "product p
                ON (p.tax_class_id = tr.tax_class_id) WHERE p.product_id = '$productId' ORDER BY tr.priority ASC"
        );

        foreach ($taxRates->rows as $taxRate) {
            $this->tax->unsetRates();
            switch ($taxRate['based']) {
                case 'shipping' :
                    $this->tax->setShippingAddress(
                        $order['shipping_country_id'],
                        $order['shipping_zone_id']
                    );
                    break;
                case 'payment':
                    $this->tax->setPaymentAddress(
                        $order['payment_country_id'],
                        $order['payment_zone_id']
                    );
                    break;
                case 'store' :
                    $this->tax->setStoreAddress(
                        $this->config->get('config_country_id'),
                        $this->config->get('config_zone_id')
                    );
                    break;
            }

            $productTaxRates = $this->tax->getRates($productPrice, $taxRate['tax_class_id']);
            if (!empty($productTaxRates)) {
                foreach ($productTaxRates as $productTaxRate) {
                    if ($productTaxRate['type'] === 'P') {
                        return $productTaxRate;
                    }
                }
            }
        }

        return [];
    }

    private function parseTaxNumber($customFields)
    {
        foreach ($customFields as $id => $value) {
            $customField = $this->model_account_custom_field->getCustomField($id);
            if (preg_match("/\bnip\b/i", $customField['name'])) {
                return $value;
            }
        }

        return null;
    }
}
